clear all,close all,clc,rng('default')
%
NumberOfVariables=3;                                                          
NumberOfVariablesFwd=1; 
SizeSystem=NumberOfVariables+NumberOfVariablesFwd;
HorizonIRF=200;                                                                    
HorizonIRFplotting=20;
%
eta=500
phi=0.5
delta=0.1
pi=1
S_z=1
rhoz=0.8
%% The Model, "Sims Canonical Form"
%
%       exchange rate                     1   e       Variables
%       price level                       2   p
%    	money supply shock                3   z
%      	expected exchange rate shock      4   
%
%      	[XR]                              1           Equations
%       [PL]                              2
%       [z]                               3
%       [FE XR]                           4
%
G0  =   [1+phi*delta/eta (1-phi*delta)/eta -1/eta -1;
        -pi*delta 1+pi*delta 0 0;
        0 0 1 0;
        0 0 0 1];                                                                             
G1  =   [0 0 0 0;
        0 1 0 0;
        0 0 rhoz 0;
        0 0 0 1];
CSI =   [0;
        0;
        1;
        0];
PI  =   [0;
        0;
        0;
        1];
%% Solving The Model, Sims' Method    
[L,O,Q,Z]=qz(G0,G1);
[L,O,Q,Z]=qzdiv(1.0001,L,O,Q,Z); 
Lambda=diag(O)./diag(L);
INST=sum(abs(Lambda)>=1);
if INST==NumberOfVariablesFwd                                         
    PIstar=Q*PI;
    L22=L(SizeSystem-INST+1:SizeSystem,SizeSystem-INST+1:SizeSystem);                                      
    PIstarx=L22\PIstar(SizeSystem-INST+1:SizeSystem,:);                                   
    CSIstar=Q*CSI;                                                  
    L11=L(1:SizeSystem-INST,1:SizeSystem-INST);
    O11=O(1:SizeSystem-INST,1:SizeSystem-INST);
    Z1=Z(:,1:SizeSystem-INST);
    %
    PIstarc=L11\PIstar(1:SizeSystem-INST,:);                                
    CSIstarc=L11\CSIstar(1:SizeSystem-INST,:);                              
    %
    CSIstarx=L22\CSIstar(SizeSystem-INST+1:SizeSystem,:);                             
    %
    B=real(Z1*(CSIstarc-PIstarc*(PIstarx\CSIstarx)));              
    F=real(Z1*(L11\O11)*Z1');
else
    MinusLogLikelihood=1.0e+100;                                       
    disp('Indeterminacy or Explosive Model')
    return
end
%% Compute The IRFs
S=zeros(SizeSystem,1);                                                                             
for tt=1:HorizonIRF
    if tt==1                                          
        S(:,tt)=B*sqrt(S_z);
    else                                             
        S(:,tt)=F*S(:,tt-1);
    end
end
%% Construct the Output Gap
S(4,1)=S(2,1);
for tt=2:HorizonIRF
    S(4,tt)=S(2,tt)-S(2,tt-1);
end
%% Plot The IRFs
figure(1)
for yy=1:NumberOfVariables+1                                
    if yy==1
        subplot(2,2,yy);plot(S(yy,1:HorizonIRFplotting),'LineWidth',5)
        title('IRF: Exchange Rate','fontsize',16)
    elseif yy==2
        subplot(2,2,yy);plot(S(yy,1:HorizonIRFplotting),'LineWidth',5)
        title('IRF: Price Level','fontsize',16)
    elseif yy==3
        subplot(2,2,yy);plot(S(yy+1,1:HorizonIRFplotting),'LineWidth',5)
        title('IRF: Output Gap','fontsize',16)
    elseif yy==4
        subplot(2,2,yy);plot(S(yy-1,1:HorizonIRFplotting),'LineWidth',5)
        title('IRF: Money Supply Shock','fontsize',16)
    else
    end
end